@@ -1,3 +1,6 @@
+from __future__ import print_function
+from __future__ import division
+from __future__ import absolute_import
 from stacker.session_cache import get_session
 
 from ...util import read_value_from_path
