@@ -10,6 +10,9 @@ Example:
     conf_value: ${xref some-fully-qualified-stack-name::SomeOutputName}
 
 """
+from __future__ import print_function
+from __future__ import division
+from __future__ import absolute_import
 from .output import deconstruct
 
 TYPE_NAME = "xref"
